@@ -228,12 +228,22 @@ function sent_conf_message_sent_do() {
    {
       if ($sent_conf_show_only_first_recip_addr)
       {
-         preg_match('/(.*?)[,; ]/', trim($send_to), $matches);
-         if (isset($matches[1]))
-            $sent_conf_message_sent_status = $matches[1] 
-               . ($sent_conf_message_style < 3 ? ', et al' : '');
+         preg_match('/\s*([\'"].*?[\'"]){0,1}(.*?)([,;]|$)/', trim($send_to), $matches);
+         if (isset($matches[2]))
+         {
+            if (isset($matches[1]))
+               $sent_conf_message_sent_status = $matches[1] . $matches[2] 
+                  . ($sent_conf_message_style < 3 ? ', et al' : '');
+            else
+               $sent_conf_message_sent_status = $matches[2] 
+                  . ($sent_conf_message_style < 3 ? ', et al' : '');
+         }
+
+         // failsafe
+         //
          else
             $sent_conf_message_sent_status = $send_to;
+
       }
       else
       {
@@ -381,18 +391,6 @@ function sent_conf_check_is_sent_do() {
    }
 
 
-   // remove leading commas/semicolons, brackets, if any
-   //
-   if (isset($sent_conf_message_sent_status) 
-          && !empty($sent_conf_message_sent_status)
-          && $sent_conf_message_sent_status != 'not_sent'
-          && $sent_conf_message_style != 'off')
-   {
-      $sent_conf_message_sent_status = preg_replace('/^(\W*)/', '', $sent_conf_message_sent_status);
-      $sent_conf_message_sent_status = preg_replace('/[<>]/', '', $sent_conf_message_sent_status);
-   }
-
-
    // if we should be showing a notification, then do it
    //
    if (isset($sent_conf_message_sent_status) 
@@ -423,12 +421,30 @@ function sent_conf_check_is_sent_do() {
       echo '<table width="100%" bgcolor="' . $color[0] 
          . '" cellpadding="0" cellspaceing="0"><tr><td>';
 
+
+      //-------------------------------------------------------------
+      // STYLE 1
+      //-------------------------------------------------------------
       if ($sent_conf_message_style == 1)
       {
          if ($sent_conf_include_recip_addr)
          {
+            $addresses = sc_parseEmailAddresses($sent_conf_message_sent_status);
+            $output = '';
+            $firstTime = TRUE;
+            foreach ($addresses as $addressAndNick) 
+            {
+               $address = trim($addressAndNick[0]);
+               if (!empty($address))
+               {
+                  if (!$firstTime) 
+                     $output .= ', ';
+                  $firstTime = FALSE;
+                  $output .= $address;
+               }
+            }
             echo '<strong>' . _("Message Sent To: ") 
-               . $sent_conf_message_sent_status;
+               . $output;
  
 
             // if any disallowed addresses from the
@@ -451,12 +467,31 @@ function sent_conf_check_is_sent_do() {
          else
             echo '<strong>' . _("Message Sent") . '</strong>';
       }
+
+
+      //-------------------------------------------------------------
+      // STYLE 2
+      //-------------------------------------------------------------
       else if ($sent_conf_message_style == 2)
       {
          if ($sent_conf_include_recip_addr)
          {
+            $addresses = sc_parseEmailAddresses($sent_conf_message_sent_status);
+            $output = '';
+            $firstTime = TRUE;
+            foreach ($addresses as $addressAndNick)
+            {
+               $address = trim($addressAndNick[0]);
+               if (!empty($address))
+               {
+                  if (!$firstTime)
+                     $output .= ', ';
+                  $firstTime = FALSE;
+                  $output .= $address;
+               }
+            }
             echo '<strong><center>' . _("Your message has been sent to ") 
-               . $sent_conf_message_sent_status;
+               . $output;
 
             // if any disallowed addresses from the
             // restrict senders plugin, display them
@@ -478,6 +513,11 @@ function sent_conf_check_is_sent_do() {
          else
             echo '<strong><center>' . _("Your message has been sent") . '</center></strong>';
       }
+
+
+      //-------------------------------------------------------------
+      // STYLE 3
+      //-------------------------------------------------------------
       else if ($sent_conf_message_style == 3)
       {
 
@@ -545,12 +585,20 @@ function sent_conf_check_is_sent_do() {
          echo '<tr><td align="center"><form name="sent_conf_form" action="../plugins/sent_confirmation/address_book_import.php" method="POST" onSubmit="if (document.sent_conf_form.elements.length == 2) { document.sent_conf_form.elements[0].checked=true; return true;} okToSubmit=false; for (i=0; i<document.sent_conf_form.elements.length; i++){if (document.sent_conf_form.elements[i].type == \'checkbox\') if (document.sent_conf_form.elements[i].checked) okToSubmit=true} if (!okToSubmit) alert(\'' . _("Please select an address to add to your address book.") . '\'); return okToSubmit"><strong>'. "\n" 
             . _("Message Sent To:") . '</strong></td></tr>'."\n"
             . '<tr><td align=center><table border=0 cellspacing=0 cellpadding=0>'."\n";
+
           
-         $addresses = preg_split('/[,;]\s*/', $sent_conf_message_sent_status);
+         // grab actual email addresses
+         //
+         $addresses = sc_parseEmailAddresses($sent_conf_message_sent_status);
+
+
+         // put addresses in the form
+         //
          $x = 0;
-         foreach ($addresses as $address)
+         foreach ($addresses as $addressAndNick)
          {
-            $address = trim($address);
+            $address = trim($addressAndNick[0]);
+            $nick = $addressAndNick[1];
             if (!empty($address))
             {
                if ($sent_conf_show_headers && strpos($address, 'Cc:') === 0)
@@ -560,7 +608,7 @@ function sent_conf_check_is_sent_do() {
                      . _("Cc:") . '</strong></td></tr><tr><td align="center">'
                      . '<table border=0 cellspacing=0 cellpadding=0>'
                      . '<tr><td><input type="checkbox" name="address' . (++$x) 
-                     . '" value="' . urlencode($address) . '">&nbsp;</td><td>'."\n"
+                     . '" value="' . urlencode($address) . '---' . urlencode($nick) . '">&nbsp;</td><td>'."\n"
                      . $address . '</td></tr>'."\n";
                }
                else if ($sent_conf_show_headers && strpos($address, 'Bcc:') === 0)
@@ -570,12 +618,12 @@ function sent_conf_check_is_sent_do() {
                      . _("Bcc:") . '</strong></td></tr><tr><td align="center">'
                      . '<table border=0 cellspacing=0 cellpadding=0>'
                      . '<tr><td><input type="checkbox" name="address' . (++$x) 
-                     . '" value="' . urlencode($address) . '">&nbsp;</td><td>'."\n"
+                     . '" value="' . urlencode($address) . '---' . urlencode($nick) . '">&nbsp;</td><td>'."\n"
                      . $address . '</td></tr>'."\n";
                }
                else
                   echo '<tr><td><input type="checkbox" name="address' . (++$x) 
-                     . '" value="' . urlencode($address) . '">&nbsp;</td><td>'."\n"
+                     . '" value="' . urlencode($address) . '---' . urlencode($nick) . '">&nbsp;</td><td>'."\n"
                      . $address . '</td></tr>'."\n";
             }
          }
@@ -709,6 +757,11 @@ function sent_conf_check_is_sent_do() {
          exit(0);
 
       }
+
+
+      //-------------------------------------------------------------
+      // STYLE 4
+      //-------------------------------------------------------------
       else if ($sent_conf_message_style == 4)
       {
 
@@ -776,12 +829,19 @@ function sent_conf_check_is_sent_do() {
          echo '<tr><td align="center"><strong>' 
             . _("Message Sent To:") . '</strong></td></tr>'
             . '<tr><td align=center><table border=0 cellspacing=0 cellpadding=0>';
+
           
-         $addresses = preg_split('/[,;]\s*/', $sent_conf_message_sent_status);
+         // grab actual email addresses
+         //
+         $addresses = sc_parseEmailAddresses($sent_conf_message_sent_status);
+
+
+         // put addresses in the form
+         //
          $x = 0;
-         foreach ($addresses as $address)
+         foreach ($addresses as $addressAndNick)
          {
-            $address = trim($address);
+            $address = trim($addressAndNick[0]);
             if (!empty($address))
             {
                if ($sent_conf_show_headers && strpos($address, 'Cc:') === 0)
@@ -947,6 +1007,116 @@ function sent_conf_check_is_sent_do() {
 
 
 
+/**
+  * Parse Out Email Addresses
+  *
+  * Takes a string of email addresses (possibly with
+  * full names, Cc or Bcc headers) and parses them
+  * into an array of real email addresses and optional
+  * nick names.
+  *
+  * @param string $emailAddressWithJunk The messy string 
+  *                                     of email addresses.
+  * @param string $header An optional string that will be 
+  *                       prepended to the first email address 
+  *                       that is parsed out. (optional)
+  *
+  */
+function sc_parseEmailAddresses($emailAddressWithJunk, $header='')
+{
+
+   // need to grab headers and do this one section at
+   // a time if headers are there (headers being CC: and BCC:)
+   //
+   if (strpos($emailAddressWithJunk, 'Cc:') !== FALSE
+    || strpos($emailAddressWithJunk, 'Bcc:') !== FALSE)
+   {
+
+      if (preg_match('/(.*)Cc:(.*)Bcc:(.*)/', $emailAddressWithJunk, $matches))
+      {
+         $toAddresses = sc_parseEmailAddresses($matches[1]);
+         $ccAddresses = sc_parseEmailAddresses($matches[2], 'Cc: ');
+         $bccAddresses = sc_parseEmailAddresses($matches[3], 'Bcc: ');
+         return array_merge($toAddresses, $ccAddresses, $bccAddresses);
+      }
+ 
+      else if (preg_match('/(.*)Cc:(.*)/', $emailAddressWithJunk, $matches))
+      {
+         $toAddresses = sc_parseEmailAddresses($matches[1]);
+         $ccAddresses = sc_parseEmailAddresses($matches[2], 'Cc: ');
+         return array_merge($toAddresses, $ccAddresses);
+      }
+
+      else if (preg_match('/(.*)Bcc:(.*)/', $emailAddressWithJunk, $matches))
+      {
+         $toAddresses = sc_parseEmailAddresses($matches[1]);
+         $bccAddresses = sc_parseEmailAddresses($matches[2], 'Bcc: ');
+         return array_merge($toAddresses, $bccAddresses);
+      }
+ 
+   }
+
+
+   // grab actual email addresses
+   //
+   preg_match_all('/\s*([\'"].*?[\'"]){0,1}(.*?)([,;]|$)/', $emailAddressWithJunk, $matches, PREG_SET_ORDER);
+   $addresses = array();
+   $isFirst = TRUE;
+   foreach ($matches as $match)
+   {
+      $nick = preg_replace('/[\'"]/', '', $match[1]);
+      if (!empty($match[2]))
+      {
+
+         // first try to get email address from within
+         // brackets (avoids malformed addresses with
+         // full name in front without quotes)
+         //
+         preg_match('/(.*)<(.*)>/', $match[2], $moreMatches);
+
+
+         // also try to grab nickname if didn't find it already
+         //
+         if (empty($nick) && !empty($moreMatches[1]))
+         {
+            $nick = trim($moreMatches[1]);
+         }
+
+
+         if (!empty($moreMatches[2]))
+         {
+            if (!empty($header) && $isFirst)
+               $moreMatches[2] = $header . $moreMatches[2];
+            $isFirst = FALSE;
+            $addresses[] = array($moreMatches[2], $nick);
+         }
+
+
+         // otherwise, it's properly formed, but might
+         // still have extraneous characters on front
+         // or back (spaces, brackets...)
+         //
+         else 
+         {
+            $address = preg_replace(array('/^(\W*)/', '/(\W*)$/'), '', $match[2]);
+            if (!empty($address))
+            {
+               if (!empty($header) && $isFirst)
+                  $address = $header . $address;
+               $isFirst = FALSE;
+               $addresses[] = array($address, $nick);
+            }
+         }
+
+      }
+   }
+
+   return $addresses;
+
+}
+
+
+
 // clear out sent indicator
 //
 function sent_conf_compose_bottom_do() {
